@@ -6,12 +6,13 @@ Created on Wed Jun 27 18:48:42 2018
 
 import logging, neoloop, os, joblib, math
 import numpy as np
-from scipy import sparse, stats
+from scipy import sparse
 from scipy.stats import poisson
-from sklearn.linear_model import HuberRegressor, LinearRegression
+from sklearn.linear_model import HuberRegressor
 from sklearn.isotonic import IsotonicRegression
-from neoloop.util import find_chrom_pre, map_coordinates
+from neoloop.util import find_chrom_pre, map_coordinates, distance_normalize, image_normalize
 from scipy.stats import spearmanr
+from scipy.ndimage import gaussian_filter
 
 log = logging.getLogger(__name__)
 
@@ -62,11 +63,36 @@ def check_increasing(x, y):
 
     return warning, increasing_bool
 
+def clean_inputs(Xi, X, Y, increasing_bool):
+
+    IR = IsotonicRegression(increasing=increasing_bool)
+    IR.fit(Xi, Y)
+    Y1 = IR.predict(Xi)
+    vi = np.where(np.diff(Y1) < 0)[0]
+    pieces = np.split(vi, np.where(np.diff(vi)!=1)[0]+1)
+    si = 0
+    for i in range(len(pieces)-1):
+        p1 = pieces[i]
+        p2 = pieces[i+1]
+        if p1.size / (p2[0] - p1[0]) > 0.5:
+            si = p1[0]
+            break
+    
+    if si / X.size > 0.3: # if more than 1/4 data discarded
+        si = vi[0]
+        if si / X.size > 0.3:
+            si = 0
+    
+    X = X[si:]
+    Y = Y[si:]
+
+    return X, Y
+
 
 class Fusion(object):
 
     def __init__(self, clr, c1, c2, p1, p2, s1, s2, sv_type, span=5000000,
-        col='sweight', trim=True, protocol='insitu'):
+        col='sweight', trim=True, protocol='insitu', expected_values=None):
 
         self.clr = clr
         self.p1 = p1
@@ -86,36 +112,26 @@ class Fusion(object):
                                 [sv_type, c1.lstrip('chr'), p1, s1, c2.lstrip('chr'), p2, s2]))
 
         self.get_matrix(span, col, trim)
-        self.load_expected()
+        if expected_values is None:
+            self.load_expected()
+        else:
+            self.expected = expected_values
     
     def load_expected(self):
 
         data_folder = os.path.join(os.path.split(neoloop.__file__)[0], 'data')
-        paths = {'dilution':{
-            10000: os.path.join(data_folder, 'gm.dilution.expected.10k.pkl'),
-            20000: os.path.join(data_folder, 'gm.dilution.expected.20k.pkl'),
-            25000: os.path.join(data_folder, 'gm.dilution.expected.25k.pkl'),
-            40000: os.path.join(data_folder, 'gm.dilution.expected.40k.pkl'),
-            50000: os.path.join(data_folder, 'gm.dilution.expected.50k.pkl')
-        },
-        'insitu':{
-            5000: os.path.join(data_folder, 'gm-300m.insitu.expected.5k.pkl'),
+        paths = {
+            5000: os.path.join(data_folder, 'gm.insitu.expected.5k.pkl'),
             10000: os.path.join(data_folder, 'gm.insitu.expected.10k.pkl'),
             20000: os.path.join(data_folder, 'gm.insitu.expected.20k.pkl'),
-            25000: os.path.join(data_folder, 'gm.insitu.expected.25k.pkl')
-        },
-        'chiapet-pol2':{
-            5000: os.path.join(data_folder, 'chiapet.mcf7.pol2.expected.5k.pkl')
-        },
-        'chiapet-ctcf':{
-            5000: os.path.join(data_folder, 'chiapet.mcf7.ctcf.expected.5k.pkl')
-        }
+            25000: os.path.join(data_folder, 'gm.insitu.expected.25k.pkl'),
+            40000: os.path.join(data_folder, 'gm.insitu.expected.40k.pkl'),
+            50000: os.path.join(data_folder, 'gm.insitu.expected.50k.pkl')
         }
         
         self.expected = {}
-        if self.protocol in paths:
-            if self.res in paths[self.protocol]:
-                self.expected = joblib.load(paths[self.protocol][self.res])
+        if self.res in paths:
+            self.expected = joblib.load(paths[self.res])
          
 
     def get_matrix(self, span, col, trim):
@@ -323,7 +339,7 @@ class Fusion(object):
         loci = loci_1 + loci_2
         loci.sort()
         if left_most:
-            b = loci[0][1]
+            b = loci[0][1] - 1
         else:
             b = loci[-1][0]
         
@@ -370,38 +386,28 @@ class Fusion(object):
         else:
             # clean the inputs by isotonic regression
             warning, increasing_bool = check_increasing(Xi, Y)
-            IR = IsotonicRegression(increasing=increasing_bool)
-            IR.fit(Xi, Y)
-            Y1 = IR.predict(Xi)
-            vi = np.where(np.diff(Y1) < 0)[0]
-            pieces = np.split(vi, np.where(np.diff(vi)!=1)[0]+1)
-            si = 0
-            for i in range(len(pieces)-1):
-                p1 = pieces[i]
-                p2 = pieces[i+1]
-                if p1.size / (p2[0] - p1[0]) > 0.5:
-                    si = p1[0]
-                    break
-            
-            if si / X.size > 0.3: # if more than 1/4 data discarded
-                si = vi[0]
-                if si / X.size > 0.3:
-                    si = 0
-            
-            X = X[si:]
-            Y = Y[si:]
+            X_clean, Y_clean = clean_inputs(Xi, X, Y, increasing_bool)
+            rscores = []
+            slopes = []
+            for X_, Y_ in ([X, Y], [X_clean, Y_clean]):
+                X_ = X_[:, np.newaxis]
+                huber = HuberRegressor().fit(X_, Y_)
+                inlier_mask = np.logical_not(huber.outliers_)
+                if inlier_mask.sum() < min_samples:
+                    rscore = 0
+                    slope = 0
+                else:
+                    sX = X_[inlier_mask]
+                    sY = Y_[inlier_mask]
+                    rscore = huber.score(sX, sY)
+                    slope = huber.coef_[0]
+                rscores.append(rscore)
+                slopes.append(slope)
 
-            X = X[:, np.newaxis]
-            huber = HuberRegressor().fit(X, Y)
-            inlier_mask = np.logical_not(huber.outliers_)
-            if inlier_mask.sum() < min_samples:
-                rscore = 0
-                slope = 0
-            else:
-                sX = X[inlier_mask]
-                sY = Y[inlier_mask]
-                rscore = huber.score(sX, sY)
-                slope = huber.coef_[0]
+            rscores = np.r_[rscores]
+            slopes = np.r_[slopes]
+            rscore = rscores.max()
+            slope = slopes[np.argmax(rscores)]
 
         return rscore, slope, warning
 
@@ -423,7 +429,7 @@ class Fusion(object):
             d_g_slopes = []
             m_g_rscores = []
             m_g_slopes = []
-            for maxdis in [400000, 1000000, 1500000]:
+            for maxdis in [200000, 400000, 500000, 1000000, 1500000, 2000000]:
                 for mink in [3]:
                     # middle
                     inter_mask = np.zeros(shape, dtype=bool)
@@ -475,7 +481,7 @@ class Fusion(object):
     
     def correct_heterozygosity(self):
 
-        ## This method assumes allele slope has been called previously
+        ## This method assumes allele slope has been called
         junc = self.fusion_point
         hcm = self.fusion_matrix.copy() # heterozygosity corrected matrix
         log.info('{0}: left slope: {1}, R^2: {2}'.format(self.name, self.u_g_s, self.u_g_r))
@@ -490,7 +496,7 @@ class Fusion(object):
             self.u_s = self.m_g_s / self.u_g_s
             self.d_s = self.m_g_s / self.d_g_s
             if (self.m_g_r > 0.6) and (self.m_g_s / ms > 0.1):
-                factor = min(1/self.m_g_s, 3/slope)
+                factor = min(1/self.m_g_s, 5/slope)
                 hcm[self.up_i:junc, junc:self.down_i] = hcm[self.up_i:junc, junc:self.down_i] * factor
 
         self.hcm = hcm
@@ -518,111 +524,85 @@ class Fusion(object):
     
 class Peakachu():
 
-    def __init__(self, matrix, lower=50000, upper=3000000, res=10000, protocol='insitu'):
+    def __init__(self, matrix, upper=4000000, res=10000, protocol='insitu'):
 
-        lower = lower // res
+        self.w = 7
         upper = upper // res
         R, C = matrix.nonzero()
         data = matrix[R, C]
-        validmask = np.isfinite(data) & (C-R+1 > lower) & (C-R < upper)
+        validmask = np.isfinite(data) & (C-R > (-2*self.w)) & (C-R < (upper+2*self.w))
         R, C, data = R[validmask], C[validmask], data[validmask]
         self.M = sparse.csr_matrix((data, (R, C)), shape=matrix.shape)
-        self.ridx, self.cidx = R, C
+        self.get_candidates(4, upper)
         self.r = res
-        if (res == 10000) or (res == 5000):
-            self.w = 5
-        else:
-            self.w = 3
-
         self.protocol = protocol
-        self.load_models()
     
-    def load_models(self):
+    def get_candidates(self, lower, upper):
 
-        data_folder = os.path.join(os.path.split(neoloop.__file__)[0], 'data')
-        paths = {
-            'dilution':{
-                10000: {
-                    'total': os.path.join(data_folder, 'dilution.10K.5.pkl')
-                },
-                20000: {
-                    'total': os.path.join(data_folder, 'dilution.20K.3.pkl')
-                },
-                25000: {
-                    'total': os.path.join(data_folder, 'dilution.25K.3.pkl')
-                },
-                40000: {
-                    'total': os.path.join(data_folder, 'dilution.40K.3.pkl')
-                },
-                50000: {
-                    'total': os.path.join(data_folder, 'dilution.50K.3.pkl')
-                }
-            },
-            'insitu': {
-                10000: {
-                    'ctcf': os.path.join(data_folder, 'insitu.ctcf.10k.pkl'),
-                    'h3k27ac': os.path.join(data_folder, 'insitu.h3k27ac.10k.pkl')
-                },
-                20000: {
-                    'ctcf': os.path.join(data_folder, 'insitu.ctcf.20k.pkl'),
-                    'h3k27ac': os.path.join(data_folder, 'insitu.h3k27ac.20k.pkl')
-                },
-                25000: {
-                    'ctcf': os.path.join(data_folder, 'insitu.ctcf.25k.pkl'),
-                    'h3k27ac': os.path.join(data_folder, 'insitu.h3k27ac.25k.pkl')
-                },
-                5000: {
-                    'ctcf': os.path.join(data_folder, 'insitu.per7.5.h3k27ac.5k.pkl'),
-                    'h3k27ac': os.path.join(data_folder, 'insitu.per7.5.h3k27ac.5k.pkl'),
-                }
-            },
-            'chiapet-pol2': {
-                5000: {
-                    'ctcf': os.path.join(data_folder, 'chiapet.mcf7.pol2.5k.pkl'),
-                    'h3k27ac': os.path.join(data_folder, 'chiapet.mcf7.pol2.5k.pkl'),
-                }
-            },
-            'chiapet-ctcf': {
-                5000: {
-                    'ctcf': os.path.join(data_folder, 'chiapet.mcf7.ctcf.5k.pkl'),
-                    'h3k27ac': os.path.join(data_folder, 'chiapet.mcf7.ctcf.5k.pkl'),
-                }
-            }
-        }
+        exp_obs = []
+        indices_obs = []
+        for i in range(lower, upper+1):
+            diag = self.M.diagonal(i)
+            if diag.size > 5:
+                exp_obs.append(diag.mean())
+                indices_obs.append(i)
+        
+        IR = IsotonicRegression(increasing=False, out_of_bounds='clip')
+        IR.fit(np.r_[indices_obs], np.r_[exp_obs])
+        d = np.arange(lower, upper+1)
+        exp = IR.predict(d)
+        expected = dict(zip(d, exp))
 
-        _models = paths[self.protocol][self.r]
+        x_arr = np.array([], dtype=int)
+        y_arr = np.array([], dtype=int)
+        idx = np.arange(self.M.shape[0])
+        for i in range(lower, upper+1):
+            diag = self.M.diagonal(i)
+            e = expected[i]
+            if (diag.size > 1) and (e > 0):
+                xi = idx[:-i]
+                yi = idx[i:]
+                diag = diag / e
+                mask = diag > 0
+                x_arr = np.r_[x_arr, xi[mask]]
+                y_arr = np.r_[y_arr, yi[mask]]
         
-        models = {w:joblib.load(_models[w]) for w in _models}
+        self.ridx, self.cidx = x_arr, y_arr
+    
+    def load_models(self, model_fil_path):
         
-        self.models = models
+        import joblib
+        
+        self.model = joblib.load(model_fil_path)
+        self.w = int((np.sqrt(self.model.feature_importances_.size) - 1) / 2)
+    
+    def load_expected(self, expected_values):
+
+        self.exp_arr = np.r_[[expected_values[i] for i in sorted(expected_values)]]
     
     def getwindow(self, coords, w):
         '''
         Extract features from center pixels.
         '''
+        coords = np.r_[coords]
+        xi, yi = coords[:,0], coords[:,1]
+        mask = (xi - w >= 0) & (yi + w + 1 <= self.M.shape[0]) & (yi - xi - 2 >= w)
+        xi, yi = xi[mask], yi[mask]
+        if xi.size < 2:
+            return [], []
+        seed = np.arange(-w, w+1)
+        delta = np.tile(seed, (seed.size, 1))
+        xxx = xi.reshape((xi.size, 1, 1)) + delta.T
+        yyy = yi.reshape((yi.size, 1, 1)) + delta
+        v = np.array(self.M[xxx.ravel(), yyy.ravel()]).ravel()
+        vvv = v.reshape((xi.size, seed.size, seed.size))
+        windows, clist = distance_normalize(vvv, self.exp_arr, xi, yi, w)
         fea = []
-        clist = []
-        for x, y in coords:
-            if (x - w < 0) or (y + w + 1 > self.M.shape[0]):
-                # suppose it's a upper-triangular matrix
-                continue
-            window = self.M[x-w:x+w+1, y-w:y+w+1].toarray()
-            if np.count_nonzero(window) < window.size * 0.2:
-                continue
-            center = window[w, w]
-            ls = window.shape[0]
-            ll = np.mean(window[ls-1-ls//4:ls, :1+ls//4])
-            if ll == 0:
-                continue
-            p2LL = center / np.mean(window[ls-1-ls//4:ls, :1+ls//4])
-            ranks = stats.rankdata(window, method='ordinal')
-            if self.protocol in ['chiapet-pol2', 'chiapet-ctcf']:
-                window = np.r_[window.ravel(), ranks]
-            else:
-                window = np.r_[window.ravel(), ranks, p2LL]
-            fea.append(window)
-            clist.append((x, y))
-        
+        for arr in windows:
+            tmp = gaussian_filter(arr, sigma=1, order=0)
+            scaled_arr = image_normalize(tmp)
+            fea.append(scaled_arr.ravel())
+
         fea = np.r_[fea]
         clist = np.r_[clist]
         
@@ -633,31 +613,29 @@ class Peakachu():
         coords = [(r, c) for r, c in zip(self.ridx, self.cidx)]
         loop_list = set()
         fea, clist = self.getwindow(coords, self.w)
-        if len(fea.shape) != 2:
+        if len(fea) < 2:
             return list(loop_list)
-
-        for k in self.models:
-            model = self.models[k]
-            probas = model.predict_proba(fea)[:, 1]
-            pM = sparse.csr_matrix((probas, (clist[:,0], clist[:,1])),
-                                shape=self.M.shape)
-            pfilter = probas >= thre
-            ri = clist[:, 0][pfilter]
-            ci = clist[:, 1][pfilter]
-            Donuts = {(i, j): self.M[i,j] for i, j in zip(ri, ci)} # total predictions
-            if not no_pool:
-                tmp = self.local_clustering(Donuts, min_count=min_count, index_map=index_map,
-                                            chains=chains)
-            else:
-                tmp = Donuts
-            for i, j in tmp:
-                loop_list.add((i, j, pM[i,j]))
+        
+        probas = self.model.predict_proba(fea)[:, 1]
+        pM = sparse.csr_matrix((probas, (clist[:,0], clist[:,1])),
+                            shape=self.M.shape)
+        pfilter = probas >= thre
+        ri = clist[:, 0][pfilter]
+        ci = clist[:, 1][pfilter]
+        Donuts = {(i, j): self.M[i,j] for i, j in zip(ri, ci)}
+        if not no_pool:
+            tmp = self.local_clustering(Donuts, min_count=min_count, index_map=index_map,
+                                        chains=chains)
+        else:
+            tmp = Donuts
+        for i, j in tmp:
+            loop_list.add((i, j, pM[i,j]))
         
         loop_list = list(loop_list)
 
         return loop_list
     
-    def _local_search(self, candidates, Donuts, r=20000, index_map=None,
+    def _local_search(self, candidates, Donuts, r=10000, index_map=None,
         chains=None):
 
         final_list = set()
@@ -684,7 +662,7 @@ class Peakachu():
         
         return final_list
     
-    def local_clustering(self, Donuts, min_count=1, r=20000, index_map=None,
+    def local_clustering(self, Donuts, min_count=2, r=15000, index_map=None,
         chains=None):
 
         byregion = {}
@@ -705,14 +683,17 @@ class Peakachu():
         for k in byregion:
             Donuts = byregion[k]
             tmp = self._local_clustering(Donuts, min_count=min_count, r=r)
+            '''
             precise_ = self._local_search(tmp, Donuts, r=r, index_map=index_map,
                                         chains=chains)
+            '''
+            precise_ = set(tmp)
             coords.update(precise_)
         
         return coords
 
 
-    def _local_clustering(self, Donuts, min_count=1, r=20000):
+    def _local_clustering(self, Donuts, min_count=1, r=15000):
 
         final_list = []
         x = np.r_[[i[0] for i in Donuts]]
@@ -722,7 +703,7 @@ class Peakachu():
 
         x_anchors = self.find_anchors(x, min_count=min_count, min_dis=r)
         y_anchors = self.find_anchors(y, min_count=min_count, min_dis=r)
-        r = max(r//self.r, 1)
+        r = max(r//self.r, 2)
         visited = set()
         lookup = set(zip(x, y))
         for x_a in x_anchors:
@@ -753,13 +734,16 @@ class Peakachu():
         
         return final_list
     
-    def find_anchors(self, pos, min_count=3, min_dis=20000, wlen=200000):
+    def find_anchors(self, pos, min_count=3, min_dis=15000, wlen=40000):
 
         from collections import Counter
         from scipy.signal import find_peaks, peak_widths
 
-        min_dis = max(min_dis//self.r, 1)
-        wlen = min(wlen//self.r, 20)
+        min_dis = max(min_dis//self.r, 2)
+        if self.r <= 10000:
+            wlen = min(wlen//self.r, 20)
+        else:
+            wlen = 4
 
         count = Counter(pos)
         refidx = range(min(count), max(count)+1)
